@@ -82,6 +82,7 @@ class PoolV1:
         coupon_range: Optional[CorePositiveRange] = None,
         coupon_type: Optional[MbsCouponType] = None,
         eligible_tba_only: Optional[bool] = None,
+        factor_date: Optional[CoreFactorDate] = None,
         issue_date_range: Optional[CoreDateRange] = None,
         issue_upb_range: Optional[CorePositiveRange] = None,
         limit: Optional[int] = None,
@@ -107,6 +108,7 @@ class PoolV1:
         _coupon_range = unwrap_or_unset(coupon_range)
         _coupon_type = unwrap_or_unset(coupon_type)
         _eligible_tba_only = unwrap_or_unset(eligible_tba_only)
+        _factor_date = unwrap_or_unset(factor_date)
         _issue_date_range = unwrap_or_unset(issue_date_range)
         _issue_upb_range = unwrap_or_unset(issue_upb_range)
         _limit = unwrap_or_unset(limit)
@@ -132,6 +134,7 @@ class PoolV1:
             coupon_range=_coupon_range,
             coupon_type=_coupon_type,
             eligible_tba_only=_eligible_tba_only,
+            factor_date=_factor_date,
             issue_date_range=_issue_date_range,
             issue_upb_range=_issue_upb_range,
             limit=_limit,
@@ -157,9 +160,12 @@ class PoolV1:
             raise ValueError("no data returned from the service")
         return response
 
-    def get(self, *, cusips: List[str]) -> PoolV1GetResponse:
+    def get(
+        self, *, cusips: List[str], factor_date: Optional[CoreFactorDate] = None
+    ) -> PoolV1GetResponse:
         _cusips = cusips
-        json_body = PoolV1GetRequest(cusips=_cusips)
+        _factor_date = unwrap_or_unset(factor_date)
+        json_body = PoolV1GetRequest(cusips=_cusips, factor_date=_factor_date)
         response = _get.sync(client=self.client, json_body=json_body)
         if isinstance(response, GoogleRpcStatus):
             raise ValueError(response.message)
