@@ -1,8 +1,9 @@
 import datetime
 import math
 import sys
+import time
 from functools import partial, wraps
-from typing import Any, Callable, Dict, List, Optional, TypeVar, Union, cast
+from typing import Any, Callable, Dict, List, Optional, Tuple, TypeVar, Union, cast
 
 if sys.version_info >= (3, 8):
     from typing import Protocol
@@ -24,15 +25,6 @@ DateT = Union[str, datetime.date, pd.Timestamp]
 KNOWN_DATE_COLUMNS = ["asOf", "factorDate", "issueDate", "maturityDate"]
 
 
-class ResponseMapping(Protocol):
-    """Mock class for helping type mapped attributes in Model objects."""
-
-    additional_keys: List[str]
-
-    def __getitem__(self, key: str) -> Any:
-        pass
-
-
 def handle_date(dt: DateT) -> CoreDate:
     """Convert date object to CoreDate."""
     _dt = pd.Timestamp(dt).date()
@@ -103,6 +95,22 @@ def _parse_exploded_date(df: pd.DataFrame, col: str) -> pd.DataFrame:
     return df.drop(to_drop, axis=1)
 
 
+def maybe_explode_columns(
+    df: pd.DataFrame, cols: Optional[List[str]] = None
+) -> pd.DataFrame:
+    if cols is None:
+        return df
+
+    for col in cols:
+        if col not in df.columns:
+            continue
+        exploded = pd.json_normalize(df[col]).rename(columns=stringcase.snakecase)
+        df.drop(col, axis=1, inplace=True)
+        df = pd.concat([df, exploded], axis=1)
+
+    return df
+
+
 def maybe_get_column(df: pd.DataFrame, col: Optional[str] = None) -> pd.DataFrame:
     if col is None or col not in df.columns:
         return df
@@ -115,39 +123,49 @@ def maybe_wide(df: pd.DataFrame, on: Optional[str] = None) -> pd.DataFrame:
     return df.unstack(on)
 
 
-def model_to_frame(model: Any, record_path: List[str], meta: List[str]) -> pd.DataFrame:
-    df = pd.json_normalize(model.to_dict(), record_path=record_path, meta=meta)
-    if len(df) == 0:
-        return df
+class Response(Protocol):
+    """Mock class for helping type mapped attributes in Model objects."""
 
-    df = df.pipe(maybe_parse_known_date_columns).rename(columns=stringcase.snakecase)
-    return df
+    additional_properties: Dict[str, Any]
+
+    def to_dict(self) -> Dict[str, Any]:
+        pass
+
+    @property
+    def additional_keys(self) -> List[str]:
+        pass
+
+    def __getitem__(self, key: str) -> Any:
+        pass
+
+    def __setitem__(self, key: str, value: Any) -> None:
+        pass
+
+    def __delitem__(self, key: str) -> None:
+        pass
+
+    def __contains__(self, key: str) -> bool:
+        pass
 
 
-def response_mapping_to_frame(
-    mapping: ResponseMapping,
-    record_path: List[str],
-    meta: List[str],
+def response_to_frame(
+    response: Response,
+    path: str,
     index_cols: List[str],
+    explode: Optional[List[str]] = None,
     col: Optional[str] = "cpr",
     wide_on: Optional[str] = None,
 ) -> Optional[pd.DataFrame]:
-    frames = [
-        model_to_frame(mapping[key], record_path, meta)
-        for key in mapping.additional_keys
-    ]
-    if frames:
-        out = pd.concat(frames, ignore_index=True)
-        if out.empty:
-            return out
-        else:
-            return (
-                out.set_index(index_cols)
-                .pipe(maybe_get_column, col=col)
-                .pipe(maybe_wide, on=wide_on)
-            )
-    else:
-        return None
+    out = nested_dict_to_frame(response.to_dict(), path=path, set_index=False)
+    if out is None:
+        return out
+    out = (
+        out.pipe(maybe_explode_columns, cols=explode)
+        .set_index(index_cols)
+        .pipe(maybe_get_column, col=col)
+        .pipe(maybe_wide, on=wide_on)
+    )
+    return out
 
 
 def nested_dict_to_frame(
@@ -156,19 +174,23 @@ def nested_dict_to_frame(
     set_index: bool = True,
 ) -> Optional[pd.DataFrame]:
     def _recurse(
-        data: Dict[str, Dict[str, Any]], keys: List[str], meta: Dict[str, Any]
+        data: Union[Dict[str, Dict[str, Any]], Dict[str, List[Any]]],
+        keys: List[Tuple[str, List[str]]],
+        meta: Dict[str, Any],
     ) -> List[Dict[str, Any]]:
         records = []
-        key = keys[0]
+        key, extra_meta_keys = keys[0]
         keys_remain = keys[1:]
-
         is_final = len(keys_remain) == 0  # on last level
         is_wild = "(*)" in key  # is a wildcard key
         key = key.replace("(*)", "")
 
         if is_wild:
+            for k in extra_meta_keys:
+                meta[k] = data[k]
+            data_dict = cast(Dict[str, Dict[str, Any]], data)
             # wildcard keys mean the dict keys should become values in an index level
-            for idx, val in data.items():
+            for idx, val in data_dict.items():
                 meta[key] = idx  # update the meta data with the key value
                 if is_final:
                     # objs from the last level should be records
@@ -177,16 +199,36 @@ def nested_dict_to_frame(
                 else:
                     # keep recursing but with updated meta
                     records.extend(_recurse(val, keys_remain, meta))
+        elif is_final and isinstance(data[key], list):
+            for k in extra_meta_keys:
+                meta[k] = data[k]
+            recs = cast(List[Any], data[key])
+            for rec in recs:
+                rec.update(meta)
+                records.append(rec)
         else:
+            for k in extra_meta_keys:
+                meta[k] = data[k]
             # regular keys should be recursed into
-            records.extend(_recurse(data[key], keys_remain, meta))
+            data_ = cast(Dict[str, Dict[str, Any]], data[key])
+            records.extend(_recurse(data_, keys_remain, meta))
 
         return records
 
-    keys = path.split("->")
+    def _parse_path(path: str) -> List[Tuple[str, List[str]]]:
+        out = []
+        tokens = path.split("->")
+        for token in tokens:
+            keys = token.split(",")
+            primary = keys[-1]
+            meta = keys[:-1]
+            out.append((primary, meta))
+        return out
+
+    keys = _parse_path(path)
     recs = _recurse(data, keys, {})
 
-    index_levels = [k.replace("(*)", "") for k in keys if "(*)" in k]
+    index_levels = [k[0].replace("(*)", "") for k in keys if "(*)" in k[0]]
 
     if recs:
         df = (
@@ -216,7 +258,7 @@ def simple_dict_to_frame(
     return df
 
 
-MAX_BATCH_SIZE = 5_000
+MAX_BATCH_SIZE = 1_000
 
 T = TypeVar("T")
 FChunker = Callable[..., Optional[T]]
@@ -235,12 +277,13 @@ def chunker(
     def _deco(func: FChunker[Optional[T]]) -> FChunker[Optional[T]]:
         @wraps(func)
         def _chunked_func(*args: Any, **kwargs: Any) -> Optional[T]:
+            chunk_size = kwargs.pop("size", size)
             items = kwargs.pop(col)
-            it = chunked(items, size)
+            it = chunked(items, chunk_size)
 
             progress: bool = kwargs.pop("progress", False)
-            if progress and len(items) > size:
-                it = tqdm.tqdm(it, total=math.ceil(len(items) / size))
+            if progress and len(items) > chunk_size:
+                it = tqdm.tqdm(it, total=math.ceil(len(items) / chunk_size))
 
             coll: List[T] = []
             for chunk in it:
